//! \brief   Implementation of the HatName window for visual_YAO.
//! \version 2009/11/09 (yyyy/mm/dd)
//! \author  MAYAKI Abdouramane & MAYOMBO Alexis.

#include "include/HatNameWindow.hpp"

//! \class HatNameWindow(string param1, QWidget *parent) : QWidget(parent)
//! \brief Implementation of the HatName window for visual_YAO.

using namespace std; //! \namespace std
using namespace yao; //! \namespace yao

/***************************************   The class constructor: HatNameWindow()  ********************************************************************/
HatNameWindow::HatNameWindow(vector <string> *aHeaderList, Table <Constant> *aConstantTable, QWidget *parent) : QWidget(parent), modeleFichiers(NULL), 
hatView(NULL), m_hatnameModel(NULL), localHeaderList(NULL), localConstantTable(NULL)

{
	setWindowTitle("Directive Hatname."); // Titre de la fenetre.
	//	move(350, 0);
	localConstantTable = aConstantTable;
	localHeaderList		= aHeaderList;
	m_labelNom1HatName	= new QLabel("Le fichier (.h): ");
	m_labelNom2HatName	= new QLabel("Recherche d'un fichier: ");
        m_comboHatName          = new QComboBox;		       //Champ de selection d'une constante
		m_comboHatName->insertItem(0, "");

		for (Table<Constant>::reverse_iterator it = localConstantTable->rbegin(); it != localConstantTable->rend(); it++) {
			m_comboHatName->insertItem(1, QString(it->getName().c_str()));
	//connect(this, SIGNAL(activated(currentText())), this, SLOT());		
		}

        m_boutonouvrir          = new QPushButton(this);	       //Déclaration du bouton "ouvrir" permettant de selectionner un fichier.
		m_boutonouvrir->setMaximumSize(35, 25);
        m_boutonAjouter         = new QPushButton("Ajouter", this);    //Déclaration du bouton "ajouter"
        m_delHatButton		= new QPushButton("Supprimer", this) ; //Déclaration du bouton "supprimmer"
		if(isHeaderListEmpty()) m_delHatButton->setDisabled(true); //m_delHatButton->setDisabled(true);
	m_quitHatName		= new QPushButton("Quitter", this);
        m_hatnameHelpButton     = new QPushButton("Aide", this);
		m_hatnameHelpButton->setMaximumSize(35, 25);
        m_spacer                = new QSpacerItem(41, 21);

        m_comboHatName->setEditable(true);
        
	m_hatnameModel = new QStandardItemModel(0, 1);//(listeFichiers);
	for (vector <string>::iterator it = localHeaderList->begin(); it != localHeaderList->end(); it++)
		m_hatnameModel->appendRow(new QStandardItem(it->c_str()));
        hatView = new QListView;
        hatView->setModel(m_hatnameModel);

	m_boutonouvrir->setIcon(QIcon("Images/icons/openfile.png")); //Affichage d'une image sur le bouton "ouvrir"
        //m_boutonouvrir->raise();
        
	m_layoutGHatName		= new QGridLayout ;
		m_layoutGHatName->addWidget(m_labelNom1HatName, 0, 1, Qt::AlignLeft);
		m_layoutGHatName->addWidget(m_comboHatName, 0, 2);
		m_layoutGHatName->addWidget(m_boutonouvrir, 0, 3, Qt::AlignLeft);	
		
		m_layoutGHatName->addWidget(hatView, 2, 1, 7, 2);

		m_layoutGHatName->addWidget(m_boutonAjouter, 3, 3);
		m_layoutGHatName->addWidget(m_delHatButton, 4, 3);
		m_layoutGHatName->addWidget(m_quitHatName, 5, 3);
		m_layoutGHatName->addWidget(m_hatnameHelpButton, 8, 3);
		setLayout(m_layoutGHatName);
		
		
	connect(m_quitHatName, SIGNAL(clicked()), this, SLOT(close()));
        connect(m_delHatButton, SIGNAL(clicked()), this, SLOT(deleteHatRow()));
        connect(m_boutonouvrir , SIGNAL(clicked()), this, SLOT(ouvrirProjetExistant()));// Permet de choisir parmi la liste de fichiers existants.
        connect(m_boutonAjouter, SIGNAL(clicked()), this, SLOT(addHatRow()));
        connect(m_hatnameHelpButton, SIGNAL(clicked()), this, SLOT(callHatHelp()));

	setFixedWidth(baseSize().width() + 345);
	//cout << "After the move() method, my coodinate is " << x() << endl;
    }

/************************************* Impémentation de la méthode getNomFichierHat() *************************************/
//! \fn string getNomFichierHat()

string HatNameWindow::getNomFichierHat() {

    return m_comboHatName->currentText().toStdString(); //! \return m_comboHatName->currentText().toStdString()
}
/************************************ Implémentation de la méthode appelEnregistrer() ************************************/
//! \fn void appelEnregistrer()

void HatNameWindow::appelEnregistrer() {

        for(QStringList::iterator it=listeFichiers.begin(); it!=listeFichiers.end(); it++)
                localHeaderList->push_back(it->toStdString());
}

/************************************** Implémentation de la méthode setFileList *****************************************/
void HatNameWindow::setFileList(string localString) {  //! \fn void setFileList(string localString)

    listeFichiers.append(localString.c_str());

}

/************************************* Implementation of the method callHatHelp() **********************************************************************/
void HatNameWindow::callHatHelp() {

QMessageBox::about(this, tr("Aide sur la directive hatname."),("<p> Première directive à déclarer obligatoirement dans tout projet Yao. Hatname permet d'intégrer en nombre limité dans le projet, les fichiers source.h. <p><u>Exemple</u>: hat_name nom_1 [nom_2 ...[nom_n]] <p> Pour plus d'informations sur la directive hatname, <a href=\"help/hatname_aide.html\">cliquez ici.</a> "));

	//cout << "In callHelp method, my coodinate is " << x() << endl;
}
/*************************************** Implementation of the method addHatRow()  ********************************************************************/
void HatNameWindow::addHatRow() {
	try {
		hatFile = getNomFichierHat();//Est prévu dans yao pour gerer les chaines de caractères.

		Constant *c; //Nous créons d'abord un pointeur de constante.

			c = localConstantTable->find(hatFile); //Ensuite on passe au pointeur le resultat de la recherche de la chaine en utilisant find.

		if (c != NULL) { //C'est alors une constante.
			string constantValue = localConstantTable->find(hatFile)->getText(); //"getText()" returns the constant value.
			constantValue.append(".h");
			m_hatnameModel->setItem(m_hatnameModel->rowCount(), new QStandardItem(QString(hatFile.c_str())));
			localHeaderList->push_back(hatFile);
			eraseDataFromLineEdit();
			m_delHatButton->setEnabled(true);
			//cout << "In addrow method, my coodinate is " << x() << endl;
		    }         

		else { //Ce n'est pas une constante. 

			for(unsigned int i=0; i < hatFile.size(); i++) {
				// stringLiteral of the YAO grammar:
				// (' ' | '\t')* ( 'a'..'z' | 'A'..'Z' | '/' | '_' | '-' | '.'| '0'..'9' )+ (' ' | '\t')*
				if(!((hatFile[i] >= 'a' && hatFile[i] <= 'z') || (hatFile[i] >= 'A' && hatFile[i] <= 'Z') ||
					(hatFile[i] >= '0' && hatFile[i] <= '9') || (hatFile[i] == '/')  ||
					(hatFile[i] == '_')  || (hatFile[i] == '.')  || (hatFile[i] == ' ') )){

					QMessageBox::about(this, tr("Erreur de saisie!"),
					       tr("<p>Vous avez saisi au moins un caractere non conforme!<p> Cliquer sur <b>ok</b> pour fermer cette fenetre."));

					return;
				}
			}

	extension = static_cast<FilePath>(hatFile).getExtension();

	if ((extension != ".h")){
	    if (extension == "")
		{
			if (hatFile.length() != 0) {
			    hatFile.append(".h");
			    QString fichier = hatFile.c_str();
	    
			    listeFichiers.append(fichier);
	    
			    localHeaderList->push_back(hatFile); 
			    m_hatnameModel->setItem(m_hatnameModel->rowCount(), new QStandardItem(QString(hatFile.c_str()))); 
			    eraseDataFromLineEdit();
			}

			return;

		}
	    else
		QMessageBox::about(this, tr("Bad extension"),
                               tr("<p>L'extension du fichier n'est pas correcte."));
             return;
      }
	else {
		 if (hatFile.length() < 2) {
			eraseDataFromLineEdit();
			return ;
		}
		else 
			if (hatFile.length() > 2)	
				{
					QString fichier =  hatFile.c_str();
					m_hatnameModel->setItem(m_hatnameModel->rowCount(), new QStandardItem(QString(hatFile.c_str())));
					localHeaderList->push_back(hatFile);
         
					eraseDataFromLineEdit();
					m_delHatButton->setEnabled(true);
				}
	   }

		}
	}
	  catch(...) {
	
		QMessageBox::critical(this, "Erreur", "Erreur dans la methode addHatRow.");

		}	
}
/***********************************************  Implementation of the method deleteHatRow() ********************************************************/
void HatNameWindow::deleteHatRow() {

         QItemSelectionModel *selection = hatView->selectionModel(); //Un ensemble sur les éléments selectionnés
         index = selection->currentIndex(); //Précision de l'élément courant selectionné
	
	if (!index.isValid())
          return;

         QVariant element = m_hatnameModel->data(index, Qt::DisplayRole);//Récupération de la valeur de l'élément courant.
         int r = index.row();// Récuperation de la ligne de l'élément courant.
         if (selection->hasSelection()) // Evite certains effets indésirables: suppression de la derniere ligne après clic sur le vide.
          {
                 m_hatnameModel->removeRows(hatView->currentIndex().row(), 1);// Le 2ème parametre indique le nombre de ligne à supprimer
                 localHeaderList->erase(localHeaderList->begin() + r);//On met à jour la structure de données en supprimant aussi l'élement.
                 //m_tableView->resizeRowToContents(c - 1); // columnCountChanged(c , c-1);
		if(isHeaderListEmpty()) m_delHatButton->setDisabled(true);
         }

	//cout << "In deleteRow method, my coodinate is " << x() << endl;
  }

/***********************************************  *Implementation of the method************************************************************************/
void HatNameWindow::eraseDataFromLineEdit() {

            m_comboHatName->clearEditText();
}

/*************************************************** Implementation of the method***********************************************************************/
void HatNameWindow::closeEvent(QCloseEvent *event){

	if (!(m_comboHatName->currentText().isEmpty()) && !(m_comboHatName->currentText().isEmpty())) 
		{     
 
			int ret = QMessageBox::warning(this, tr("Confirmation de fermeture"), tr("Voulez-vous vraiment quitter?"),
				QMessageBox::Yes | QMessageBox::Default, QMessageBox::Cancel | QMessageBox::Escape);
 
			 if (ret == QMessageBox::Yes) 
				event->accept(); //cout << " test qui marche!" << endl;
         //         event->accept();
			else
				event->ignore();
		}

	emit hatWindowIsClosed(2);
}

/************************************************* Implementation of the method isHeaderListEmpty() **********************************************/
bool HatNameWindow::isHeaderListEmpty() {
 
         if (localHeaderList->empty())
                 return true;
         else
                 return false;
 }

/***************************************************Implementation of the method***********************************************************************/
void HatNameWindow::ouvrirProjetExistant() {

  boiteDialogue = new QFileDialog(this); //Crée une boite de dialogue
     cheminFichier = boiteDialogue->getOpenFileName(this, tr("Insertion d'un fichier"), "./projects", tr("*.*")); //Retourne le chemin du fichier.
  m_comboHatName->setEditText(cheminFichier); //Affichage du resultat précédent dans le champ de saisie du nom du fichier.
}

/*************************************************** Implementation of the method getTheHeaderList() **************************************************/
vector<string> *HatNameWindow::getTheHeaderList() {

    localHeaderList->push_back(getLocalProjectName());
    return localHeaderList;
 }

/********************** Implementation of the method setTheHeaderList(vector<string> *theHeaderList) **************************************************/
void HatNameWindow::setTheHeaderList(vector<string> *theHeaderList) {

	localHeaderList = theHeaderList;
}

/***************************************** Implementation of the method setLocalProjectName(string name) *****************************************/
void HatNameWindow::setLocalProjectName(string name) {

    localProjectName = name;

}

/****************************************************** Implementation of the method getLocalProjectName() *******************************************/
string HatNameWindow::getLocalProjectName() {

    return localProjectName;

}


/******************************************************* Implementation of the method getHatList() ****************************************************/
QStringList HatNameWindow::getHatList() {

    return listeFichiers;

 }
/******************************************************************************************************************************************************/
//void HatNameWindow::updateComb
/******************************************************** Implementation of the destructor ************************************************************/
HatNameWindow::~HatNameWindow() {

//	delete hatView;
}
/****************************************************************************************************************************************************/
/************************************************** The end of the file. ****************************************************************************/
/****************************************************************************************************************************************************/
